@@ -1,3 +1,5 @@
+'use strict';
+
 /**
  * Module dependencies.
  */
@@ -25,7 +27,7 @@ module.exports = Test;
  * @api public
  */
 
-function Test(app, method, path, host) {
+function Test(app, method, path) {
   Request.call(this, method.toUpperCase(), path);
   this.redirects(0);
   this.buffer();
@@ -33,7 +35,7 @@ function Test(app, method, path, host) {
   this._asserts = [];
   this.url = typeof app === 'string'
     ? app + path
-    : this.serverAddress(app, path, host);
+    : this.serverAddress(app, path);
 }
 
 /**
@@ -51,7 +53,7 @@ Object.setPrototypeOf(Test.prototype, Request.prototype);
  * @api private
  */
 
-Test.prototype.serverAddress = function(app, path, host) {
+Test.prototype.serverAddress = function(app, path) {
   var addr = app.address();
   var port;
   var protocol;
@@ -59,9 +61,35 @@ Test.prototype.serverAddress = function(app, path, host) {
   if (!addr) this._server = app.listen(0);
   port = app.address().port;
   protocol = app instanceof https.Server ? 'https' : 'http';
-  return protocol + '://' + (host || '127.0.0.1') + ':' + port + path;
+  return protocol + '://127.0.0.1:' + port + path;
 };
 
+/**
+ * Wraps an assert function into another.
+ * The wrapper function edit the stack trace of any assertion error, prepending a more useful stack to it.
+ *
+ * @param {Function} assertFn
+ * @returns {Function} wrapped assert function
+ */
+
+function wrapAssertFn(assertFn) {
+  var savedStack = new Error().stack.split('\n').slice(3);
+
+  return function(res) {
+    var badStack;
+    var err = assertFn(res);
+    if (err instanceof Error && err.stack) {
+      badStack = err.stack.replace(err.message, '').split('\n').slice(1);
+      err.stack = [err.toString()]
+        .concat(savedStack)
+        .concat('----')
+        .concat(badStack)
+        .join('\n');
+    }
+    return err;
+  };
+}
+
 /**
  * Expectations:
  *
@@ -81,7 +109,7 @@ Test.prototype.serverAddress = function(app, path, host) {
 Test.prototype.expect = function(a, b, c) {
   // callback
   if (typeof a === 'function') {
-    this._asserts.push(a);
+    this._asserts.push(wrapAssertFn(a));
     return this;
   }
   if (typeof b === 'function') this.end(b);
@@ -89,22 +117,22 @@ Test.prototype.expect = function(a, b, c) {
 
   // status
   if (typeof a === 'number') {
-    this._asserts.push(this._assertStatus.bind(this, a));
+    this._asserts.push(wrapAssertFn(this._assertStatus.bind(this, a)));
     // body
     if (typeof b !== 'function' && arguments.length > 1) {
-      this._asserts.push(this._assertBody.bind(this, b));
+      this._asserts.push(wrapAssertFn(this._assertBody.bind(this, b)));
     }
     return this;
   }
 
   // header field
   if (typeof b === 'string' || typeof b === 'number' || b instanceof RegExp) {
-    this._asserts.push(this._assertHeader.bind(this, { name: '' + a, value: b }));
+    this._asserts.push(wrapAssertFn(this._assertHeader.bind(this, { name: '' + a, value: b })));
     return this;
   }
 
   // body
-  this._asserts.push(this._assertBody.bind(this, a));
+  this._asserts.push(wrapAssertFn(this._assertBody.bind(this, a)));
 
   return this;
 };
@@ -145,7 +173,7 @@ Test.prototype.end = function(fn) {
  */
 
 Test.prototype.assert = function(resError, res, fn) {
-  var error;
+  var errorObj;
   var i;
 
   // check for unexpected network errors or server not running/reachable errors
@@ -162,23 +190,23 @@ Test.prototype.assert = function(resError, res, fn) {
   if (!res && resError) {
     if (resError instanceof Error && resError.syscall === 'connect'
         && Object.getOwnPropertyNames(sysErrors).indexOf(resError.code) >= 0) {
-      error = new Error(resError.code + ': ' + sysErrors[resError.code]);
+      errorObj = new Error(resError.code + ': ' + sysErrors[resError.code]);
     } else {
-      error = resError;
+      errorObj = resError;
     }
   }
 
   // asserts
-  for (i = 0; i < this._asserts.length && !error; i += 1) {
-    error = this._assertFunction(this._asserts[i], res);
+  for (i = 0; i < this._asserts.length && !errorObj; i += 1) {
+    errorObj = this._assertFunction(this._asserts[i], res);
   }
 
   // set unexpected superagent error if no other error has occurred.
-  if (!error && resError instanceof Error && (!res || resError.status !== res.status)) {
-    error = resError;
+  if (!errorObj && resError instanceof Error && (!res || resError.status !== res.status)) {
+    errorObj = resError;
   }
 
-  fn.call(this, error || null, res);
+  fn.call(this, errorObj || null, res);
 };
 
 /**
